from datetime import datetime
from typing import Optional

from pyorthanc.async_client import AsyncOrthanc
from pyorthanc.client import Orthanc


def make_datetime_from_dicom_date(date: str, time: str = None) -> Optional[datetime]:
    """Attempt to decode date"""
    try:
        return datetime(
            year=int(date[:4]),
            month=int(date[4:6]),
            day=int(date[6:8]),
            hour=int(time[:2]),
            minute=int(time[2:4]),
            second=int(time[4:6])
        )
    except (ValueError, TypeError):
        try:
            return datetime(
                year=int(date[:4]),
                month=int(date[4:6]),
                day=int(date[6:8]),
            )
        except (ValueError, TypeError):
            return None


def async_to_sync(orthanc: AsyncOrthanc) -> Orthanc:
    sync_orthanc = Orthanc(url=orthanc.url)
    sync_orthanc._auth = orthanc.auth

    return sync_orthanc


def sync_to_async(orthanc: Orthanc) -> AsyncOrthanc:
    async_orthanc = AsyncOrthanc(url=orthanc.url)
    async_orthanc._auth = orthanc.auth

    return async_orthanc



import asyncio
import copy
import os
import warnings
from typing import List, Dict, Callable, Optional, Tuple, Union

from pyorthanc.study import Study
from pyorthanc.series import Series
from pyorthanc.instance import Instance
from pyorthanc.async_client import AsyncOrthanc
from pyorthanc.client import Orthanc
from pyorthanc.patient import Patient
from pyorthanc.util import async_to_sync


def build_patient_forest(
        orthanc: Orthanc,
        patient_filter: Optional[Callable] = None,
        study_filter: Optional[Callable] = None,
        series_filter: Optional[Callable] = None) -> List[Patient]:
    warnings.warn(
        'Function "build_patient_forest" is deprecated and will be removed in a future release. '
        'Please use "find" instead',
        DeprecationWarning,
        stacklevel=2
    )
    return find(orthanc, patient_filter, study_filter, series_filter)


def find(orthanc_url: str,
         auth: Tuple[str, str] = None,
         async_mode: bool = False,
         patient_filter: Optional[Callable] = None,
         study_filter: Optional[Callable] = None,
         series_filter: Optional[Callable] = None) -> List[Patient]:
    """Find desired patients/Study/Series/Instance in an Orthanc server

    This function builds a series of tree structure.
    Each tree correspond to a patient. The layers in the
    tree correspond to:

    `Patient -> Studies -> Series -> Instances`

    Parameters
    ----------
    orthanc_url
        Orthanc server url.
    auth
        Tuple of username and password, if needed.
    async_mode
    patient_filter
        Patient filter (e.g. lambda patient: patient.id_ == '03HDQ99*')
    study_filter
        Study filter (e.g. lambda study: study.study_id == '*pros*')
    series_filter
        Series filter (e.g. lambda series: series.modality == 'SR')

    Returns
    -------
    List[Patient]
        List of patient.
    """
    if async_mode:
        if auth:
            orthanc = AsyncOrthanc(url=orthanc_url, username=auth[0], password=auth[1])
        else:
            orthanc = AsyncOrthanc(url=orthanc_url)

        patients = asyncio.run(
            _async_find(
                orthanc,
                patient_filter,
                study_filter,
                series_filter
            )
        )

        return trim_patient(patients)

    if auth:
        orthanc = Orthanc(url=orthanc_url, username=auth[0], password=auth[1])
    else:
        orthanc = Orthanc(url=orthanc_url)

    patient_identifiers = orthanc.get_patients()
    patients = []

    for patient_id in patient_identifiers:  # This ID is the Orthanc's ID, and not the PatientID
        patients.append(_build_patient(
            patient_id,
            orthanc,
            patient_filter,
            study_filter,
            series_filter
        ))

    return trim_patient(patients)


async def _async_find(
        orthanc: AsyncOrthanc,
        patient_filter: Optional[Callable] = None,
        study_filter: Optional[Callable] = None,
        series_filter: Optional[Callable] = None) -> List[Patient]:
    patient_identifiers = await orthanc.get_patients()
    tasks = []

    for patient_id in patient_identifiers:  # This ID is the Orthanc's ID, and not the PatientID
        task = asyncio.create_task(
            _async_build_patient(
                patient_id,
                orthanc,
                patient_filter,
                study_filter,
                series_filter
            )
        )
        tasks.append(task)

    patients = await asyncio.gather(*tasks)

    return patients


def _build_patient(
        patient_identifier: str,
        orthanc: Orthanc,
        patient_filter: Optional[Callable],
        study_filter: Optional[Callable],
        series_filter: Optional[Callable]) -> Patient:
    patient = Patient(patient_identifier, orthanc)

    if patient_filter is not None:
        if not patient_filter(patient):
            return patient

    study_information = orthanc.get_patients_id_studies(patient_identifier)
    patient._studies = [_build_study(i, orthanc, study_filter, series_filter) for i in study_information]

    return patient


async def _async_build_patient(
        patient_identifier: str,
        orthanc: AsyncOrthanc,
        patient_filter: Optional[Callable],
        study_filter: Optional[Callable],
        series_filter: Optional[Callable]) -> Patient:
    patient = Patient(patient_identifier, async_to_sync(orthanc))

    if patient_filter is not None:
        if not patient_filter(patient):
            return patient

    study_information = await orthanc.get_patients_id_studies(patient_identifier)

    tasks = []
    for info in study_information:
        task = asyncio.create_task(_async_build_study(info, orthanc, study_filter, series_filter))
        tasks.append(task)

    patient._studies = await asyncio.gather(*tasks)

    return patient


def _build_study(
        study_information: Dict,
        orthanc: Orthanc,
        study_filter: Optional[Callable],
        series_filter: Optional[Callable]) -> Study:
    study = Study(study_information['ID'], orthanc, study_information)

    if study_filter is not None:
        if not study_filter(study):
            return study

    series_information = orthanc.get_studies_id_series(study_information['ID'])
    study._series = [_build_series(i, orthanc, series_filter) for i in series_information]

    return study


async def _async_build_study(
        study_information: Dict,
        async_orthanc: AsyncOrthanc,
        study_filter: Optional[Callable],
        series_filter: Optional[Callable]) -> Study:
    study = Study(study_information['ID'], async_to_sync(async_orthanc), study_information)

    if study_filter is not None:
        if not study_filter(study):
            return study

    series_information = await async_orthanc.get_studies_id_series(study_information['ID'])

    tasks = []
    for info in series_information:
        task = asyncio.create_task(_async_build_series(info, async_orthanc, series_filter))
        tasks.append(task)

    study._series = await asyncio.gather(*tasks)

    return study


def _build_series(
        series_information: Dict,
        orthanc: Orthanc,
        series_filter: Optional[Callable]) -> Series:
    series = Series(series_information['ID'], orthanc, series_information)

    if series_filter is not None:
        if not series_filter(series):
            return series

    instance_information = orthanc.get_series_id_instances(series_information['ID'])
    series._instances = [Instance(i['ID'], orthanc, i) for i in instance_information]

    return series


async def _async_build_series(
        series_information: Dict,
        orthanc: AsyncOrthanc,
        series_filter: Optional[Callable]) -> Series:
    series = Series(series_information['ID'], async_to_sync(orthanc), series_information)

    if series_filter is not None:
        if not series_filter(series):
            return series

    instance_information = await orthanc.get_series_id_instances(series_information['ID'])
    series._instances = [Instance(i['ID'], async_to_sync(orthanc), i) for i in instance_information]

    return series


def trim_patient(patients: List[Patient]) -> List[Patient]:
    """Trim Patient forest (list of patients)

    Parameters
    ----------
    patients
        Patient forest.

    Returns
    -------
    List[Patient]
        Pruned patient forest.
    """
    for patient in patients:
        patient.remove_empty_studies()

    patients = [p for p in patients if p.studies != []]

    return list(patients)


def retrieve_and_write_patients(patients: List[Patient], path: str) -> None:
    """Retrieve and write patients to given path

    Parameters
    ----------
    patients
        Patient forest.
    path
        Path where you want to write the files.
    """
    os.makedirs(path, exist_ok=True)
    for patient in patients:
        retrieve_patient(patient, path)


def retrieve_patient(patient: Patient, path: str) -> None:
    patient_path = _make_patient_path(path, patient.id_)
    os.makedirs(patient_path, exist_ok=True)

    for study in patient.studies:
        retrieve_study(study, patient_path)


def retrieve_study(study: Study, patient_path: str) -> None:
    study_path = _make_study_path(patient_path, study.id_)
    os.makedirs(study_path, exist_ok=True)

    for series in study.series:
        retrieve_series(series, study_path)


def retrieve_series(series: Series, study_path: str) -> None:
    series_path = _make_series_path(study_path, series.modality)
    os.makedirs(series_path, exist_ok=True)

    for instance in series.instances:
        retrieve_instance(instance, series_path)


def retrieve_instance(instance: Instance, series_path) -> None:
    path = os.path.join(series_path, instance.uid + '.dcm')

    dicom_file_bytes = instance.get_dicom_file_content()

    with open(path, 'wb') as file_handler:
        file_handler.write(dicom_file_bytes)


def _make_patient_path(path: str, patient_id: str) -> str:
    patient_directories = os.listdir(path)

    if patient_id.strip() == '':
        patient_id = 'anonymous-patient'
        return os.path.join(path, _make_path_name(patient_id, patient_directories))

    return os.path.join(path, patient_id)


def _make_study_path(patient_path: str, study_id: str) -> str:
    study_directories = os.listdir(patient_path)

    if study_id.strip() == '':
        study_id = 'anonymous-study'

    return os.path.join(patient_path, _make_path_name(study_id, study_directories))


def _make_series_path(study_path: str, modality: str) -> str:
    series_directories = os.listdir(study_path)

    return os.path.join(study_path, _make_path_name(modality, series_directories))


def _make_path_name(name: str, directories: List[str], increment: int = 1, has_increment: bool = False) -> str:
    if not has_increment:
        name = f'{name}-{increment}'
        has_increment = True
    else:
        name = '-'.join(name.split('-')[:-1])

    if name in directories:
        return _make_path_name(name, directories, increment + 1, has_increment)

    return name



from pyorthanc.async_client import AsyncOrthanc
from pyorthanc.client import Orthanc
from pyorthanc.remote import RemoteModality
from pyorthanc.patient import Patient
from pyorthanc.study import Study
from pyorthanc.series import Series
from pyorthanc.instance import Instance
from pyorthanc.filtering import build_patient_forest, find, trim_patient, retrieve_and_write_patients


__all__ = [
    'AsyncOrthanc',
    'Orthanc',
    'RemoteModality',
    'Patient',
    'Study',
    'Series',
    'Instance',
    'build_patient_forest',
    'trim_patient',
    'retrieve_and_write_patients',
    'find',
]



from datetime import datetime
from typing import List, Dict

from pyorthanc.client import Orthanc
from pyorthanc.series import Series
from pyorthanc.util import make_datetime_from_dicom_date


class Study:
    """Represent a study that is in an Orthanc server

    This object has many getters that allow the user to retrieve metadata
    or the entire DICOM file of the Series
    """

    def __init__(
            self,
            study_id: str,
            client: Orthanc,
            study_information: Dict = None) -> None:
        """Constructor

        Parameters
        ----------
        study_id
            Orthanc study identifier.
        client
            Orthanc object.
        study_information
            Dictionary of study's information.
        """
        self.client = client

        self.id_ = study_id
        self.information = study_information

        self._series: List[Series] = []

    @property
    def identifier(self) -> str:
        """Get Study Orthanc's identifier

        Returns
        -------
        str
            Study identifier
        """
        return self.id_

    def get_main_information(self) -> Dict:
        """Get Study information

        Returns
        -------
        Dict
            Dictionary of study information
        """
        if self.information is None:
            self.information = self.client.get_studies_id(self.id_)

        return self.information

    @property
    def referring_physician_name(self) -> str:
        """Get referring physician name

        Returns
        -------
        str
            Referring physician Name.
        """
        return self.get_main_information()['MainDicomTags']['ReferringPhysicianName']

    @property
    def date(self) -> datetime:
        """Get study date

        The date have precision to the second (if available).

        Returns
        -------
        datetime
            Study date
        """
        date_string = self.get_main_information()['MainDicomTags']['StudyDate']
        time_string = self.get_main_information()['MainDicomTags']['StudyTime']

        return make_datetime_from_dicom_date(date_string, time_string)

    @property
    def study_id(self) -> str:
        """Get Study ID

        Returns
        -------
        str
            Study ID
        """
        try:
            return self.get_main_information()['MainDicomTags']['StudyID']
        except KeyError:
            return ''

    @property
    def uid(self) -> str:
        """Get StudyInstanceUID

        Returns
        -------
        str
            StudyInstanceUID
        """
        return self.get_main_information()['MainDicomTags']['StudyInstanceUID']

    @property
    def patient_identifier(self) -> str:
        """Get the Orthanc identifier of the parent patient

        Returns
        -------
        str
            Parent patient's identifier.
        """
        return self.get_main_information()['ParentPatient']

    @property
    def patient_information(self) -> Dict:
        """Get patient information

        Returns
        -------
        Dict
            Patient general information.
        """
        return self.get_main_information()['PatientMainDicomTags']

    @property
    def series(self) -> List[Series]:
        """Get Study series

        Returns
        -------
        List[Series]
            List of study's Series
        """
        return self._series

    def anonymize(self, remove: List = None, replace: Dict = None, keep: List = None, force: bool = False) -> 'Study':
        """Anonymize Study

        If no error has been raise, then it creates a new anonymous study.
        Documentation: https://book.orthanc-server.com/users/anonymization.html

        Parameters
        ----------
        remove
            List of tag to remove
        replace
            Dictionary of {tag: new_content}
        keep
            List of tag to keep unchanged
        force
            Some tags can't be change without forcing it (e.g. PatientID) for security reason

        Returns
        -------
        Study
            A new anonymous Study.
        """
        remove = [] if remove is None else remove
        replace = {} if replace is None else replace
        keep = [] if keep is None else keep

        anonymous_study = self.client.post_studies_id_anonymize(
            self.id_,
            json={'Remove': remove, 'Replace': replace, 'Keep': keep, 'Force': force}
        )

        return Study(anonymous_study['ID'], self.client)

    def get_zip(self) -> bytes:
        """Get the bytes of the zip file

        Get the .zip file.

        Returns
        -------
        bytes
            Bytes of Zip file of the study.

        Examples
        --------
        >>> from pyorthanc import Orthanc, Study
        >>> a_study = Study(
        ...     'STUDY_IDENTIFIER',
        ...     Orthanc('http://localhost:8042')
        ... )
        >>> bytes_content = a_study.get_zip()
        >>> with open('study_zip_file_path.zip', 'wb') as file_handler:
        ...     file_handler.write(bytes_content)

        """
        return self.client.get_studies_id_archive(self.id_)

    def build_series(self) -> None:
        """Build a list of the study's series."""
        series_information = self.client.get_studies_id_series(self.id_)
        self._series = [Series(i['ID'], self.client) for i in series_information]
        for series in self._series:
            series.build_instances()

    def remove_empty_series(self) -> None:
        """Delete empty series."""
        self._series = list(filter(
            lambda series: series.instances != [],
            self._series
        ))

    def __repr__(self):
        return f'Study(StudyId={self.study_id}, identifier={self.id_})'



from typing import List, Dict

from pyorthanc.instance import Instance
from pyorthanc.client import Orthanc


class Series:
    """Represent an series that is in an Orthanc server

    This object has many getters that allow the user to retrieve metadata
    or the entire DICOM file of the Series
    """

    def __init__(
            self,
            series_id: str,
            client: Orthanc,
            series_information: Dict = None) -> None:
        """Constructor

        Parameters
        ----------
        series_id
            Orthanc series identifier.
        client
            Orthanc object.
        series_information
            Dictionary of series information.
        """
        self.client = client

        self.id_ = series_id
        self.information = series_information

        self._instances: List[Instance] = []

    @property
    def instances(self) -> List[Instance]:
        """Get series instance

        Returns
        -------
        List[Instance]
            List of the series' Instance.
        """
        return self._instances

    @property
    def identifier(self) -> str:
        """Get series identifier

        Returns
        -------
        str
            Series identifier.
        """
        return self.id_

    @property
    def uid(self) -> str:
        """Get SeriesInstanceUID

        Returns
        -------
        str
            SeriesInstanceUID
        """
        return self.get_main_information()['MainDicomTags']['SeriesInstanceUID']

    def get_main_information(self) -> Dict:
        """Get series main information

        Returns
        -------
        Dict
            Dictionary of series main information.
        """
        if self.information is None:
            self.information = self.client.get_series_id(self.id_)

        return self.information

    @property
    def manufacturer(self) -> str:
        """Get the manufacturer

        Returns
        -------
        str
            The manufacturer.
        """
        return self.get_main_information()['MainDicomTags']['Manufacturer']

    @property
    def study_identifier(self) -> str:
        """Get the parent study identifier

        Returns
        -------
        str
            The parent study identifier.
        """
        return self.get_main_information()['ParentStudy']

    @property
    def modality(self) -> str:
        """Get series modality

        Returns
        -------
        str
            Series modality.
        """
        return self.get_main_information()['MainDicomTags']['Modality']

    @property
    def series_number(self) -> str:
        """Get series number

        Returns
        -------
        str
            Series number.
        """
        return self.get_main_information()['MainDicomTags']['SeriesNumber']

    def anonymize(self, remove: List = None, replace: Dict = None, keep: List = None, force: bool = False) -> 'Series':
        """Anonymize Series

        If no error has been raise, then it creates a new anonymous series.
        Documentation: https://book.orthanc-server.com/users/anonymization.html

        Parameters
        ----------
        remove
            List of tag to remove
        replace
            Dictionary of {tag: new_content}
        keep
            List of tag to keep unchanged
        force
            Some tags can't be change without forcing it (e.g. PatientID) for security reason

        Returns
        -------
        Series
            A new anonymous Series.
        """
        remove = [] if remove is None else remove
        replace = {} if replace is None else replace
        keep = [] if keep is None else keep

        anonymous_series = self.client.post_series_id_anonymize(
            self.id_,
            json={'Remove': remove, 'Replace': replace, 'Keep': keep, 'Force': force}
        )

        return Series(anonymous_series['ID'], self.client)

    def get_zip(self) -> bytes:
        """Get the bytes of the zip file

        Get the .zip file.

        Returns
        -------
        bytes
            Bytes of Zip file of the series.

        Examples
        --------
        >>> from pyorthanc import Orthanc, Series
        >>> a_series = Series(
        ...     'SERIES_IDENTIFIER',
        ...     Orthanc('http://localhost:8042')
        ... )
        >>> bytes_content = a_series.get_zip()
        >>> with open('series_zip_file_path.zip', 'wb') as file_handler:
        ...     file_handler.write(bytes_content)

        """
        return self.client.get_series_id_archive(self.id_)

    def build_instances(self) -> None:
        """Build a list of the series' instances."""

        instance_ids = self.client.get_series_id_instances(self.id_)
        self._instances = [Instance(i['ID'], self.client) for i in instance_ids]

    def __repr__(self):
        return f'Series(identifier={self.id_})'



from typing import List, Dict

from pyorthanc.study import Study
from pyorthanc.client import Orthanc


class Patient:
    """Represent a Patient that is in an Orthanc server

    This object has many getters that allow the user to retrieve metadata
    or the entire DICOM file of the Patient
    """

    def __init__(
            self,
            patient_id: str,
            client: Orthanc,
            patient_information: Dict = None) -> None:
        """Constructor

        Parameters
        ----------
        patient_id
            Orthanc patient identifier.
        client
            Orthanc object.
        patient_information
            Dictionary of patient's information.
        """
        self.client = client

        self.id_ = patient_id
        self.information = patient_information

        self._studies: List[Study] = []

    @property
    def identifier(self) -> str:
        """Get patient identifier

        Returns
        -------
        str
            Patient identifier
        """
        return self.id_

    def get_main_information(self) -> Dict:
        """Get Patient information

        Returns
        -------
        Dict
            Dictionary of patient main information.
        """
        if self.information is None:
            self.information = self.client.get_patients_id(self.id_)

        return self.information

    @property
    def patient_id(self) -> str:
        """Get patient ID

        Returns
        -------
        str
            Patient ID
        """
        return self.get_main_information()['MainDicomTags']['PatientID']

    @property
    def name(self) -> str:
        """Get patient name

        Returns
        -------
        str
            Patient name
        """
        return self.get_main_information()['MainDicomTags']['PatientName']

    @property
    def sex(self) -> str:
        """Get patient sex

        Returns
        -------
        str
            Patient sex
        """
        return self.get_main_information()['MainDicomTags']['PatientSex']

    def get_zip(self) -> bytes:
        """Get the bytes of the zip file

        Get the .zip file.

        Returns
        -------
        bytes
            Bytes of Zip file of the patient.

        Examples
        --------
        >>> from pyorthanc import Orthanc, Patient
        >>> a_patient = Patient(
        ...     'A_PATIENT_IDENTIFIER',
        ...     Orthanc('http://localhost:8042')
        ... )
        >>> bytes_content = a_patient.get_zip()
        >>> with open('patient_zip_file_path.zip', 'wb') as file_handler:
        ...     file_handler.write(bytes_content)

        """
        return self.client.get_patients_id_archive(self.id_)

    def get_patient_module(self, simplify: bool = False, short: bool = False) -> Dict:
        """Get patient module in a simplified version

        The method returns the DICOM patient module
        (PatientName, PatientID, PatientBirthDate, ...)

        Parameters
        ----------
        simplify
            Get the simplified version of the tags
        short
            Get the short version of the tags

        Returns
        -------
        Dict
            DICOM Patient module.
        """
        if simplify and not short:
            params = {'simplify': True}
        elif short and not simplify:
            params = {'short': True}
        elif simplify and short:
            raise ValueError('simplify and short can\'t be both True')
        else:
            params = {}

        return dict(self.client.get_patients_id_module(
            self.id_,
            params=params
        ))

    def is_protected(self) -> bool:
        """Get if patient is protected against recycling

        Protection against recycling: False means unprotected, True protected.

        Returns
        -------
        bool
            False means unprotected, True means protected.
        """
        return '1' == self.client.get_patients_id_protected(self.id_)

    def set_to_protected(self):
        """Set patient to protected state

        Returns
        -------
        None
            Nothing.
        """
        # As of version 1.11.1, the Orthanc OPEN API file has missing information
        self.client._put(
            f'{self.client.url}/patients/{self.id_}/protected',
            json=1
        )

    def set_to_unprotected(self):
        """Set patient to unprotected state

        Returns
        -------
        None
            Nothing.
        """
        # As of version 1.11.1, the Orthanc OPEN API file has missing information
        self.client._put(
            f'{self.client.url}/patients/{self.id_}/protected',
            json=0
        )

    @property
    def studies(self) -> List[Study]:
        """Get patient's studies

        Returns
        -------
        List[Study]
            List of the patient's studies
        """
        return self._studies

    def build_studies(self) -> None:
        """Build a list of the patient's studies
        """
        studies_information = self.client.get_patients_id_studies(self.id_)

        self._studies = [Study(i['ID'], self.client) for i in studies_information]
        for study in self._studies:
            study.build_series()

    def anonymize(self, remove: List = None, replace: Dict = None, keep: List = None, force: bool = False) -> 'Patient':
        """Anonymize patient

        If no error has been raise, then it creates a new anonymous patient.
        Documentation: https://book.orthanc-server.com/users/anonymization.html

        Parameters
        ----------
        remove
            List of tag to remove
        replace
            Dictionary of {tag: new_content}
        keep
            List of tag to keep unchanged
        force
            Some tags can't be change without forcing it (e.g. PatientID) for security reason

        Returns
        -------
        Patient
            A New anonymous patient.
        """
        remove = [] if remove is None else remove
        replace = {} if replace is None else replace
        keep = [] if keep is None else keep

        anonymous_patient = self.client.post_patients_id_anonymize(
            self.id_,
            json={'Remove': remove, 'Replace': replace, 'Keep': keep, 'Force': force}
        )

        return Patient(anonymous_patient['ID'], self.client)

    def __str__(self):
        return f'Patient(PatientID={self.patient_id}, identifier={self.identifier})'

    def remove_empty_studies(self) -> None:
        """Delete empty studies."""
        for study in self._studies:
            study.remove_empty_series()

        self._studies = list(filter(
            lambda s: s.series != [], self._studies
        ))



from typing import Dict

import httpx

from pyorthanc.client import Orthanc


class RemoteModality:
    """Wrapper around Orthanc API when dealing with a (remote) modality.
    """

    def __init__(self, client: Orthanc, modality: str) -> None:
        """Constructor

        Parameters
        ----------
        client
            Orthanc object.
        modality
            Remote modality.
        """
        self.client = client
        self.modality = modality

    def echo(self) -> bool:
        """C-Echo to remote modality

        Returns
        -------
        bool
            True if C-Echo succeeded.
        """
        try:
            self.client.post_modalities_id_echo(self.modality)
            return True

        except httpx.HTTPError:
            return False

    def query(self, data: Dict) -> Dict:
        """C-Find (Querying with data)

        Parameters
        ----------
        data
            Dictionary to send in the body of request.

        Returns
        -------
        Dict
            Dictionary with keys {'ID': '...', 'path': '...'}

        Examples
        -------
        >>> data = {'Level': 'Study',
        ...         'Query': {
        ...             'PatientID':'03HD*',
        ...             'StudyDescription':'*Chest*',
        ...             'PatientName':''
        ...         }
        ... }

        >>> remote_modality = RemoteModality(
        ...     client=Orthanc('http://localhost:8042'),
        ...     modality='sample'
        ... )

        >>> remote_modality.query(data)
        """
        return dict(self.client.post_modalities_id_query(self.modality, json=data))

    def move(self, query_identifier: str, cmove_data: Dict) -> Dict:
        """C-Move query results to another modality

        C-Move SCU: Send all the results to another modality whose AET is in the body

        Parameters
        ----------
        query_identifier
            Query identifier.
        cmove_data
            Ex. {'TargetAet': 'target_modality_name', "Synchronous": False}

        Returns
        -------
        Dict
            Orthanc Response (probably a Dictionary)

        Examples
        --------
        >>> remote_modality = RemoteModality(Orthanc('http://localhost:8042'), 'modality')
        >>> query_id = remote_modality.query(
        ...     data={'Level': 'Series',
        ...           'Query': {'PatientID': '',
        ...                     'Modality':'SR'}})

        >>> remote_modality.move(
        ...     query_identifier=query_id['ID'],
        ...     cmove_data={'TargetAet': 'TARGETAET'}
        ... )

        """
        return dict(self.client.post_queries_id_retrieve(query_identifier, json=cmove_data))

    def store(self, instance_or_series_id: str) -> Dict:
        """Store series or instance to remote modality.

        Parameters
        ----------
        instance_or_series_id
            Instance or Series Orthanc identifier.

        Returns
        -------
        Dict
            Information related to the C-Store operation.
        """
        return dict(self.client.post_modalities_id_store(
            self.modality,
            json=instance_or_series_id
        ))

    def get_query_answers(self) -> Dict:
        answers = {}

        for query_id in self.client.get_queries():
            for answer_id in self.client.get_queries_id_answers(query_id):
                answers[query_id] = self.client.get_queries_id_answers_index_content(query_id, answer_id)

        return answers





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyorthanc', 'pyorthanc.deprecated']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.23.0,<0.24.0', 'pydicom>=2.3.0,<3.0.0']

setup_kwargs = {
    'name': 'pyorthanc',
    'version': '1.11.2',
    'description': 'Orthanc REST API python wrapper with additional utilities',
    'long_description': "PyOrthanc\n=========\nPython library that wraps the Orthanc REST API and facilitates the manipulation\nof data with several cool utilities.\n\n\nBreaking changes\n----------------\nPyOrthanc has been rewritten almost entirely from `0.*.*`.\nThe new `Orthanc` client is now automatically generated from `https://api.orthanc-server.com/`.\nThe version `*.*.` of PyOrthanc will follow Orthanc version \n(e.g. `pyorthanc 1.11.*` is generated from the API specification of Orthanc `1.11.*`).\n\nThis means that the method names of the `Orthanc` objects from `PyOrthanc 0.*` are no longer the same.\nYou can still use the old client with\n```python\nfrom pyorthanc.deprecated.client import Orthanc\n```\n\nNote that due to automatic generation some method names may be less clear.\nHowever, the automatic generation allows PyOrthanc to cover all the routes of the API of Orthanc.\n\n\n\nInstallation\n------------\n```sh\n$ pip install pyorthanc\n```\n\nExample of usage\n----------------\nBe sure that Orthanc is running. The default URL (if running locally) is `http://localhost:8042`.\n\n#### Getting access to patients, studies, series and instances information:\n\n```python\nfrom pyorthanc import Orthanc\n\northanc = Orthanc('http://localhost:8042', username='username', password='password')\n\n# To get patients identifier and main information\npatients_identifiers = orthanc.get_patients()\n\nfor patient_identifier in patients_identifiers:\n   # To get patient information\n   patient_info = orthanc.get_patients_id(patient_identifier)\n\n   patient_name = patient_info['MainDicomTags']['PatientName']\n   ...\n   study_identifiers = patient_info['Studies']\n\n# To get patient's studies identifier and main information\nfor study_identifier in study_identifiers:\n   # To get Study info\n   study_info = orthanc.get_studies_id(study_identifier)\n\n   study_date = study_info['MainDicomTags']['StudyDate']\n   ...\n   series_identifiers = study_info['Series']\n\n# To get study's series identifier and main information\nfor series_identifier in series_identifiers:\n   # Get series info\n   series_info = orthanc.get_series_id(series_identifier)\n\n   modality = series_info['MainDicomTags']['Modality']\n   ...\n   instance_identifiers = series_info['Instances']\n\n# and so on ...\nfor instance_identifier in instance_identifiers:\n   instance_info = orthanc.get_instances_id(instance_identifier)\n   ...\n```\n\n#### Find patients with certain characteristics in an Orthanc instance:\nEach patient is a tree. Layers in each tree have the following structure \n`Patient` -> `Study` -> `Series` -> `Instance`\nthat correspond to the provided filter functions.\n\n```python\nfrom pyorthanc import find\n\npatients = find(\n    orthanc_url='http://localhost:8042/',\n    auth=('username', 'password'),\n    series_filter=lambda s: s.modality == 'RTDOSE'  # Optional: filter with pyorthanc.Series object\n)\n\nfor patient in patients:\n   patient_info = patient.get_main_information()\n   patient.id_   # Access PatientID\n   patient.name  # Access PatientName\n   \n   patient.get_zip() # DICOM files' content in bytes\n   \n   anonymized_patient_1 = patient.anonymize()  # New patient that was anonymized by Orthanc\n   anonymized_patient_2 = patient.anonymize(\n      keep=['PatientName'],   # You can keep/remove/replace the DICOM tags you want\n      replace={'PatientID': 'TheNewPatientID'},\n      remove=['ReferringPhysicianName'],\n      force=True  # Needed when changing PatientID/StudyInstanceUID/SeriesInstanceUID/SOPInstanceUID\n   )  \n   ...\n\n   for study in patient.studies:\n      study.date  # Date as a datetime object\n      study.referring_physician_name\n      ...\n\n      for series in study.series:\n         series.modality  # Should be 'RTDOSE' because of the series_filter parameters\n         ...\n```\n\n\n#### Upload DICOM files to Orthanc:\n\n```python\nfrom pyorthanc import Orthanc\n\northanc = Orthanc('http://localhost:8042', 'username', 'password')\n\nwith open('A_DICOM_INSTANCE_PATH.dcm', 'rb') as file:\n   orthanc.post_instances(file.read())\n```\n\n#### Getting list of connected remote modalities:\n```python\nfrom pyorthanc import Orthanc\n\northanc = Orthanc('http://localhost:8042', 'username', 'password')\n\northanc.get_modalities()\n```\n\n#### Query (C-Find) and Retrieve (C-Move) from remote modality:\n\n```python\nfrom pyorthanc import RemoteModality, Orthanc\n\northanc = Orthanc('http://localhost:8042', 'username', 'password')\n\nmodality = RemoteModality(orthanc, 'modality')\n\n# Query (C-Find) on modality\ndata = {'Level': 'Study', 'Query': {'PatientID': '*'}}\nquery_response = modality.query(data=data)\n\nanswer = modality.get_query_answers()[query_response['ID']]\nprint(answer)\n\n# Retrieve (C-Move) results of query on a target modality (AET)\nmodality.move(query_response['ID'], {'TargetAet': 'target_modality'})\n```\n\n#### Anonymize patient:\n```python\nfrom pyorthanc import Orthanc, Patient\n\northanc = Orthanc('http://localhost:8042', 'username', 'password')\n\npatient_identifier = orthanc.get_patients()[0]\n\nanonymized_patient = Patient(patient_identifier, orthanc).anonymize(\n    keep=['PatientName'],   # You can keep/remove/replace the DICOM tags you want\n    replace={'PatientID': 'TheNewPatientID'},\n    remove=['ReferringPhysicianName'],\n    force=True  # Needed when changing PatientID/StudyInstanceUID/SeriesInstanceUID/SOPInstanceUID\n)\n# Or directly with\northanc.post_patients_id_anonymize(patient_identifier)\n\n# result is: (you can retrieve DICOM file from ID)\n# {'ID': 'dd41f2f1-24838e1e-f01746fc-9715072f-189eb0a2',\n#  'Path': '/patients/dd41f2f1-24838e1e-f01746fc-9715072f-189eb0a2',\n#  'PatientID': 'dd41f2f1-24838e1e-f01746fc-9715072f-189eb0a2',\n#  'Type': 'Patient'}\n```\n\n## Citation\nIf you publish using PyOrthanc, we kindly ask that you credit us. PyOrthanc can be found on Zenodo :\nhttps://zenodo.org/record/7086219 .\n\n\n## Contributing\nYou can contribute to this project with the following steps:\n1. First, fork the project on Github \n2. Clone the project\n   ```shell\n   git clone https://github.com/<your-github-username>/pyorthanc\n   cd pyorthanc\n   ```\n3. Enter the project and create a poetry environment \n   (this project use the [poetry](https://python-poetry.org/) for dependency management)\n   ```shell\n   peotry install \n   ```\n4. Make a new git branch where you will apply the changes\n   ```shell\n   git checkout -b your-branch-name\n   ```\n   Now you can make your changes\n5. Once done, `git add`, `git commit` and `git push` the changes.\n6. Make a Pull Request from your branch to the https://github.com/ulaval-rs/pyorthanc.\n",
    'author': 'Gabriel Couture',
    'author_email': 'gacou54@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
